#!/usr/bin/env python
# Python 2.*

import os.path
from gcodeParser import *
from termcolor import colored
import time

class App:
    def __init__(self):
        self.intersections = 0
        self.path = None
        self.model = None
    
    def main(self):
        import sys
        if len(sys.argv) <= 1:
            print colored('Please provide a .gcode file path','yellow')
            print
        else:
            print(('-' * 71))
            self.path = sys.argv[1]
            self.load()
            self.evaluate()
            print(('-' * 71))
            print
        
    def load(self):    
        print "Loading file %s..."%repr(self.path)
        parser = GcodeParser(False)
        self.model = parser.parseFile(self.path)
        print colored("File %s successfully parsed"%repr(self.path),'green')
        print colored("Number of layers: %s"%len(self.model.layers),'blue')
        print colored("Number of segments: %s"%len(self.model.segments),'blue')
        
    def evaluate(self):
        print
        print "Evaluating intersections..."
        t1 = time.time()
        previousLayer = None            
        for layer in self.model.layers:
            if (previousLayer):
                previousSegment = None
                for segment in layer.segments:
                    if (previousSegment and segment.style == 'extrude'):
                        previousUnderSegment = None
                        for underSegment in previousLayer.segments:
                            if (previousUnderSegment and underSegment.style == 'extrude'): 
                                if (self.isCrossing(previousSegment, segment, previousUnderSegment, underSegment)):
                                    self.intersections += 1
                            previousUnderSegment = underSegment
                    previousSegment = segment
            previousLayer = layer
        t2 = time.time()
        print colored("Evaluated in %0.2fs" %(t2-t1), 'green')
        print colored('Number of intersections: %s'%self.intersections, 'blue')
        
    def isCrossing(self, A0, B0, A1, B1):
        p = Vector(A0)
        q = Vector(A0, B0)
        r = Vector(A1)
        s = Vector(A1, B1)
        mv = MathVector()
        det = mv.xNorm(q, s)
        if (q.isNull() or s.isNull() or det == 0):
            return False        
        m = mv.minus(r, p)
        t = mv.xNorm(m, s) / det
        u = mv.xNorm(m, q) / det
        return (t > 0 and t < 1 and u > 0 and u < 1)
     
        
class Vector:
    def __init__(self, startPoint = None, endPoint = None):
        if not (startPoint):
            self.coords = {"X": 0,
                           "Y": 0}
        elif not (endPoint):
            self.coords = {"X": startPoint.coords["X"],
                           "Y": startPoint.coords["Y"]}
        else: 
            self.coords = {"X": endPoint.coords["X"] - startPoint.coords["X"], 
                           "Y": endPoint.coords["Y"] - startPoint.coords["Y"]}  
    
    def isNull(self):
        return (self.coords["X"] == 0 and self.coords["Y"] == 0)
    
    
class MathVector:
    def xNorm(self, u, v):
        return u.coords["X"] * v.coords["Y"] - v.coords["X"] * u.coords["Y"]
    
    def minus(self, u, v):
        result = Vector()
        result.coords["X"] = u.coords["X"] - v.coords["X"] 
        result.coords["Y"] = u.coords["Y"] - v.coords["Y"]
        return result
     
     
App().main()
